# Ovsem ostatni autentifikacni moduly, je zrejme nutne mit vyple - napriklad mod_auth_sys.so
# Vhodne pro rychlost je asi mit mudul naloadovany pri startu Apache "PerlModule Apache::MyAuthen"

package Apache::MyAuthen;

use strict;
use warnings;
use lib ("/home/web-mycompany/www/lib");
$Apache::MyAuthen::VERSION = '0.14opteron';

use Apache::Access ();
use Apache::RequestUtil ();
use Apache::Const ':common';
use Apache::RequestRec ();
use Apache::Connection;
use Apache::Log;

use My;

#==== funkce ktera urcuje zda je pravo
sub handler {
    my $r = shift;

    # nastaveni promene prostredi, urcujici pozdeji o ktery jde web
    $ENV{DOCUMENT_ROOT}=$r->document_root;

    my($res, $sent_pwd) = $r->get_basic_auth_pw;
    return $res if $res; #decline if not Basic

    my $name = $r->user();

    if ($name eq "") {
        $r->note_basic_auth_failure;
        $r->log_error("Apache::MyAuthen - no username given", $r->uri);
        return AUTH_REQUIRED;
    }

    #- zjisteni skutecneho zakodovaneho hesla - je nejake nespolehlive, kdyz priprava dotazu je nad fci
    my ($user, $passwd);
    my $dbhpravo=My->dbhpravo;
    my $sth_shadow=$dbhpravo->prepare(q{
       select
          Uzivatele.heslo
       from
          Prava.Uzivatele
       where
          Uzivatele.web=? and
          Uzivatele.uziv=?
    });
    $sth_shadow->execute(My->web,$name);
    my $shadow=$sth_shadow->fetchrow_hashref;
    $sth_shadow->finish;

    if(length($$shadow{heslo})) {
       $user=$name;
       $passwd=$$shadow{heslo};
    }

    unless ($user) {
        $r->note_basic_auth_failure;
        $r->log_error("Apache::MyAuthen - user $name: unknown or bad passwd", $r->uri);
        return AUTH_REQUIRED;
    }

    # ziskani nazvu okruhu
    my $okruh;
    if(rindex($r->uri,"?")!=-1) {
        $okruh=substr( $r->uri, 0, rindex($r->uri,"?") );
    }
    else {
        $okruh=$r->uri;
    }
    # dodano po realizaci nph-suideperlu pres rewrite pravidla (bylo nutne v Apache 2.0.47)
    # !!! po prechodu na puvodni koncept  s AddHandler (ci AddType) to zrus
    if($okruh=~/^\/cgi-bin\/nph-suideperl(.+)$/) {
       $okruh=$1;
    }

    # pri pristupu z localhostu neni pro vstup do okruhu /share/ vyzadovano, aby heslo bylo ok
    if($r->connection->get_remote_host eq "127.0.0.1" and $okruh=~/^\/share\//) {
       return OK;
    }
    # jinak je nutne, aby heslo bylo dobre zadane
    elsif(crypt($sent_pwd, $passwd) eq $passwd) {
        # u pocitace adm, natvrdo dodavam pred jmeno okruhu retezec "/dok"
        if(My->local_hostname eq "adm") {
           $okruh="/dok".$okruh;
        }
        if(My->pravo($user,$okruh)) {
           return OK;
        }
        else {
           $r->note_basic_auth_failure;
           $r->log_error("Apache::MyAuthen - user $name: no privileges", $r->uri);
           return AUTH_REQUIRED;
        }
    }
    # u spatne zadaneho hesla je vyzadano znovu prihlaseni
    else {
        $r->note_basic_auth_failure;
        $r->log_error("Apache::MyAuthen - user $name: bad password", $r->uri);
        return AUTH_REQUIRED;
    }

    return OK;
}

1;

