package Forum;

my $pkg="Forum";

use strict;
use My;
use MyXML;
use Output;
use Form;
use Info;
use Date::Calc;
use Date::Format;

my $dbh=My->dbh;


#=== Neketere globalni promene jen pro vnitrni uziti v tomto balicku

my %punc_color=(
   "0" => "gray",
   "1" => "#333333",
   "2" => "green",
   "3" => "red"
);
my %punc_highlight=(
   "2" => "#bbffbb",
   "3" => "#ffbbbb"
);


#=== Fce definujici zalezitosti kolem prstredi

#--- Fce vraci v hashi fora, kde je povine vnucen (1) ci zakazan (0) jejich odber (samozrejme pokud ma uzivatel na ne pravo)
sub odber {
   my $odber;
   my $sth=$dbh->prepare(q{
      select
         Forum_odber.forum
      from
         Komunikace.Forum_odber
      where
         Forum_odber.uzivzapis=?
   });
   # Prislusna fora jsou pro kazdeho uzivatele automaticky prihlasena a neni je mozne odhlasit
   $sth->execute("+");
   while(my $ref=$sth->fetchrow_hashref) {
      $$odber{$$ref{forum}}=1;
   }
   $sth->finish;
   # Prislusna fora jsou pro kazdeho uzivatele automaticky neprihlasene a neni je mozne prihlasit
   $sth->execute("-");
   while(my $ref=$sth->fetchrow_hashref) {
      $$odber{$$ref{forum}}=0;
   }
   $sth->finish;
   return $odber;
}

#--- Root cesta k webovemu stromy - vyuziva se pro fora k dokumentu
sub webpath {
   return "/home/web-mycompany/www";
}

#--- Cesta, kde se nachazi programy fora ve webovem stromu
sub forumpath {
   return "/Komunikace/Forum/";
}


#=== Elementarni zobrazovaci fce

#--- Zobrazeni vybranych ci vsech prispevku ---
sub show_message {
   my ($pkg_p, $tO,$forum,$mode,$limit0,$limit1,$uziv,$puncmin,@id)=@_;
   ($limit0,$limit1)=$pkg->set_limit($limit0,$limit1);
   if(!defined(@id)) {
      my $query_string=q{
         select
            Forum.id
         from
            Komunikace.Forum
      };
      if(length($puncmin)) {
         $query_string.=q{
            inner join Komunikace.Forum_punc on (Forum.id=Forum_punc.id and Forum.forum=Forum_punc.forum)
         };
      }
      $query_string.=q{
         where
            Forum.forum=?
      };
      if($mode=~/thread/) {
         $query_string.=q{
            and Forum.supid=?
         };   
      }
      if(length($uziv) and $uziv ne "*") {
         $query_string.=q{
            and Forum.uzivzapis=?
         };
      }
      if(length($puncmin)) {
         $query_string.=q{
            and Forum_punc.uzivzapis=?
            and Forum_punc.punc>=?
         };
      }
      $query_string.=q{
         order by
            Forum.datzapis desc
         limit
            ?,?
      };
      my $sth=$dbh->prepare($query_string);
      my $param=0;
      $sth->bind_param(++$param, $forum);
      if($mode=~/thread/) {
         $sth->bind_param(++$param, "");
      }
      if(length($uziv) and $uziv ne "*") {
         $sth->bind_param(++$param, $uziv);
      }
      if(length($puncmin)) {
         $sth->bind_param(++$param, My->remote_user);
         $sth->bind_param(++$param, $puncmin);
      }
      $sth->bind_param(++$param, $limit0-1);
      $sth->bind_param(++$param, $limit1);
      $sth->execute;
      while(my $ref=$sth->fetchrow_hashref) {
         push(@id,$$ref{id});
      }
      $sth->finish;
   }
   my $t2O;
   if($mode=~/index/) {
      my $mode_pom;
      if($mode=~/thread/) {
         $mode_pom="thread";
      }
      else {
         $mode_pom="time";
      }
      my ($formO,$divO)=Form->form_val_list(
         $tO,
         {
            forum => { type => "hidden", value => $forum },
            limit0 => { type => "hidden", value => $limit0 },
            limit1 => { type => "hidden", value => $limit1 },
            mode => { type => "hidden", value => $mode_pom },
            uziv => { type => "hidden", value => $uziv },
            puncmin => { type => "hidden", value => $puncmin },
            _layout => [ [ { names => ["forum","limit0","limit1","mode","uziv","puncmin"] } ] ]
         },
         ["Vypsat ve / oznaen"]
      );
      $divO->removeAttribute("class"); # specialni vzhledova uprava
      my $tableO=$divO->aT(
         "table",
         {
            width => "620px",
            cellpadding => "1px",
            cellspacing => "0px",
            style => "font-size: 75%; border-top: 1px solid #d1c8b7; border-left: 1px solid #d1c8b7; border-right: 1px solid #d1c8b7;"
         }
      );
      my $colgroupO=$tableO->aT("colgroup");
      $colgroupO->aT("col",{"valign"=>"top"});
      foreach my $col (2..6) {
         $colgroupO->aT(
            "col",
            {
               "valign" => "top",
               "style" => "padding-left: 5px; padding-right: 5px;"
            }
         );
      }
      my $trO=$tableO->aT("tr", { style => "background-color: #f7eedd;" } );
      foreach my $colname ("\x{a0}","Punc","\x{a0}","Datum","Autor","Pedmt") {
         $trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7; font-weight : bold;" }, $colname );
      }
      $t2O=$tableO;

   }
   else {
      $t2O=$tO;
   }
   my $i=0;
   foreach my $id (@id) {
      $i=$pkg->rekurze_print_message($t2O,$forum,$id,$mode,$i,0);
      if($i>=$limit1) {
         last;
      }
   }
   if($i>=$limit1) {
      $tO->aT("p");
      $tO->aT("a", { href => $pkg->forumpath . "show.phtml?_idcs=" . encode_uri(get_env("idcs")) . "&limit0=" . ($limit0+$limit1) . "&limit1=$limit1" }, "dalch $limit1 pspvk");
   }
}

#--- Fce vola tisk prispevku - pokud jde o mode thread fce se bude rekurzivne volat dokud nevytiskne cele vlakno (thread)
sub rekurze_print_message {
   my ($pkg_p, $tO,$forum,$id,$mode,$i,$rekurze)=@_;
   ++$i;
   my $message=$pkg->read_message($forum,$id);
   if($mode=~/index/) {
      $pkg->print_message_index($tO,$message,$rekurze);
   }
   else {
      $pkg->print_message($tO,$message);
   }
   if($mode=~/thread/) {
      my $sth=$dbh->prepare(q{
         select
            Forum.id
         from
            Komunikace.Forum
         where
            Forum.forum=? and
            Forum.supid=?
         order by
            Forum.datzapis asc
      });
      $sth->execute($forum,$$message{id});
      while(my $ref=$sth->fetchrow_hashref) {
         if($mode!~/index/) {
            my $dirO=$tO->aT("dir", { "style" => "margin-top: 0px; margin-bottom: 0px;" });
            $i=$pkg->rekurze_print_message($dirO,$forum,$$ref{id},$mode,$i,$rekurze+1);
         }
         else {
            $i=$pkg->rekurze_print_message($tO,$forum,$$ref{id},$mode,$i,$rekurze+1);
         }
      }
      $sth->finish;
   }
   return $i;
}

#--- Tisk konkretniho prispevku jen zakladni informace na jeden radek - vsechny informace jsou v promene $message ---
sub print_message_index {
   my ($pkg_p, $tO,$message,$rekurze)=@_;
   my $style;
   if($rekurze>0) {
      $style="background-color: #fffbf4;";
   }
   else {
      $style="background-color: #f7eedd;";
   }
   my $trO=$tO->aT("tr",{style=>$style});
   my $tdO=$trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7;" } );
   $tdO->aT(
      "input",
      {
         type => "checkbox",
         name => "id",
         value => $$message{id}
      }
   );
   my $tdO=$trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7;" } );
   $pkg->print_punc($tdO,$$message{punc},$$message{punc});
   my $tdO=$trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7;" } );
   my $nobrO=$tdO->aT("nobr");
   $nobrO->aT(undef,{}," (");
   $nobrO->aT(undef,{},sprintf("%.1f",$$message{punc_avg}));
   $nobrO->aT(undef,{},"/");
   $nobrO->aT(undef,{},$$message{punc_count});
   $nobrO->aT(undef,{},")");
   my $tdO=$trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7;" } );
   my $nobrO=$tdO->aT("nobr");
   $nobrO->aT(undef,{},substr($$message{datzapis},0,10));
   my $tdO=$trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7;" } );
   my $nobrO=$tdO->aT("nobr");
   ## varovani - fce nize je celkem pomala na volani nekolikrat za sebou
   my $Edatum=get_env("datum");
   set_env("datum",$$message{datzapis});
   my $info_oc=Info->uziv($$message{uzivzapis},{uziv=>1});
   set_env("datum",$Edatum);
   $nobrO->aT(undef,{},$$info_oc{jmeno} . " " . $$info_oc{prijmeni});
   my $padding= 5 + ($rekurze*25);
   my $tdO=$trO->aT("td", { style => "border-bottom: 1px solid #d1c8b7; padding-left: " . $padding . "px;" } );
   $pkg->print_subject($tdO,$message,1); # zmena dne 2001-11-20 - puvodne: $pkg->subject_print($message,0); - ta 0 znamena nepodbarvovat subjekty v zobrazeni seznam - ted je tedy podbarvujes
}

#--- Tisk konkretniho prispevku - vsechny informace jsou v promene $message ---
sub print_message {
   my ($pkg_p, $tO,$message)=@_;
   my $pO=$tO->aT("p");
   my $tbO=$pO->aT(
      "table",
      {
         "width" => "550px",
         "cellspacing" => "0"
      }
   );
   my $trO=$tbO->aT("tr",{"style"=>"background-color: #E5E5E5;"});
   my $tdO=$trO->aT("td");
      my $tb2O=$tdO->aT("table",{"width"=>"100%","cellspacing"=>"0"});
      my $tr2O=$tb2O->aT("tr");
      my $td2O=$tr2O->aT("td");
         my $tb3O=$td2O->aT("table",{"cellspacing"=>"0"});
         my $cg3O=$tb3O->aT("colgroup");
         $cg3O->aT("col",{"valign"=>"top"});
         $cg3O->aT("col",{"valign"=>"top"});
         my $tr3O=$tb3O->aT("tr");
         $tr3O->aT("td");
         my $tr3O=$tb3O->aT("tr");
         my $td3O=$tr3O->aT("td");
         $td3O->aT("i",{},"Autor: ");
         my $td3O=$tr3O->aT("td");
         ## varovani - fce nize je celkem pomala na volani nekolikrat za sebou
         my $Edatum=get_env("datum");
         set_env("datum",$$message{datzapis});
         my $info_oc=Info->uziv($$message{uzivzapis},{uziv=>1,zamfce=>1});
         set_env("datum",$Edatum);
         $td3O->aT("a",
            {
               "style" => "text-decoration: none; color: #000000;",
               "href" => "mailto:" . $$info_oc{mail}
            },
            $$info_oc{jmeno} . " " . $$info_oc{prijmeni}
         );
         if(length($$info_oc{str}) or length($$info_oc{funkce_nazev})) {
            $td3O->aT(undef, {}, " (");
            if(length($$info_oc{str})) {
               $td3O->aT(undef, {}, $$info_oc{str});
            }
            if(length($$info_oc{str}) and length($$info_oc{funkce_nazev})) {
               $td3O->aT(undef, {}, " - ");
            }
            if(length($$info_oc{funkce_nazev})) {
               $td3O->aT(undef, {}, $$info_oc{funkce_nazev});
            }
            $td3O->aT(undef, {}, ")");
         }
         my $tr3O=$tb3O->aT("tr");
         my $td3O=$tr3O->aT("td");
         $td3O->aT("i",{},"Pedmt: ");
         my $td3O=$tr3O->aT("td");
         $pkg->print_subject($td3O,$message,1);
         my $tr3O=$tb3O->aT("tr");
         my $td3O=$tr3O->aT("td");
         $td3O->aT("i",{},"Datum: ");
         $tr3O->aT("td",{},$$message{datzapis});
      my $td2O=$tr2O->aT("td", { valign => "top", align => "right" });
      $td2O->aT(
         "a",
         {
            href => $pkg->forumpath . "add.phtml?forum=" . encode_uri($$message{forum}) . "&re=$$message{id}",
            class => "button"
         },
         "Reagovat"
      );
      my $divO=$td2O->aT("div", { style => "text-align: right; margin-right: 5px" });
      if($$message{is_thread}) {
         $divO->aT(
            "a",
            {
               href => $pkg->forumpath . "show.phtml?forum=" . encode_uri($$message{forum}) . "&id=$$message{id}&mode=thread",
               style => "text-decoration: none"
            },
            "obsah"
         );
         $divO->aT(undef,{},", ");
         $divO->aT(
            "a",
            {
               href => $pkg->forumpath . "show.phtml?forum=" . encode_uri($$message{forum}) . "&id=$$message{id}&mode=indexthread",
               style => "text-decoration: none"
            },
            "seznam"
         );
         $divO->aT(undef,{}," ");
         $divO->aT(undef,{},"nit");
      }
      my $divO=$td2O->aT("div", { style => "float: right; text-align: left;", title => "punc" });
      my $nobrO=$divO->aT("nobr");
      foreach my $punc (0..3) {
         my $puncO;
         if(defined(get_env("idcs"))) {
            $puncO=$nobrO->aT("a", { href => $pkg->forumpath . "show.phtml?_idcs=" . encode_uri(get_env("idcs")) . "&setid=" . encode_uri($$message{id}) . "&setpunc=" . encode_uri($punc) } );
         }
         else {
            $puncO=$nobrO->aT("span");
         }
         $pkg->print_punc($puncO,$punc,$$message{punc});
         $nobrO->aT(undef,{}," ");
      }
      $nobrO->aT(
         undef,
         {},
         " (" . sprintf("%.1f",$$message{punc_avg}) . "/" . $$message{punc_count} . ")"
      );
   my $trO=$tbO->aT("tr",{"style"=>"background-color: #EEEEEE"});
   my $tdO=$trO->aT("td", { colspan => "2", style => "padding: 20px;" } );
   if($$message{format} ne 'html') {
      foreach my $radek (split(/\n/,$$message{telo})) {
         $tdO->aT(undef,{},$radek);
         $tdO->aT("br");
      }
   }
   else {
      $tdO->aNchilds_fromStringHTML($$message{telo});
   }
}

#--- Nacteni prispevku ---------------
sub read_message {
   my ($pkg_p,$forum,$id)=@_;
   # dotaz vracejici konkretni prispevek
   my $sth_message=$dbh->prepare(q{
      select
         Forum.*
      from
         Komunikace.Forum
      where
         Forum.forum=? and
         Forum.id=?
   });
   # dotaz vracejici informace o punci a precteni prispevku u konkretniho uzivatele
   my $sth_messagepri=$dbh->prepare(q{
      select
         Forum_punc.*
      from
         Komunikace.Forum_punc
      where
         Forum_punc.forum=? and
         Forum_punc.id=? and
         Forum_punc.uzivzapis=?
   });
   # dotaz zjistujici prumerny punc
   my $sth_messagepriavg=$dbh->prepare(q{
      select
         avg(Forum_punc.punc) as punc_avg,
         count(*) as punc_count
      from
         Komunikace.Forum_punc
      where
         Forum_punc.forum=? and
         Forum_punc.id=?
   });
   # dotaz zjistujici zda existuje nejaky prispevek v niti vyse
   my $sth_messagesupid=$dbh->prepare(q{
      select
         Forum.forum
      from
         Komunikace.Forum
      where
         Forum.forum=? and
         Forum.supid=?
   });
   $sth_message->execute($forum,$id);
   my $read_message=$sth_message->fetchrow_hashref;
   $sth_message->finish;
   $sth_messagepri->execute($forum,$id,My->remote_user);
   my $ref=$sth_messagepri->fetchrow_hashref;
   $sth_messagepri->finish;
   $$read_message{punc}=$$ref{punc};
   $sth_messagepriavg->execute($forum,$id);
   my $ref=$sth_messagepriavg->fetchrow_hashref;
   $sth_messagepriavg->finish;
   $$read_message{punc_avg}=$$ref{punc_avg};
   $$read_message{punc_count}=$$ref{punc_count};
   # ma se zobrazovat moznost zobrazeni dane nite
   $$read_message{is_thread}=0;
   if(length($$read_message{supid})) {
      $$read_message{is_thread}=1;
   }
   else {
      $sth_messagesupid->execute($forum,$id);
      if($sth_messagesupid->rows>0) {
         $$read_message{is_thread}=1;
      }
      $sth_messagesupid->finish;
   }
   return $read_message;
}

#-- fce hledajici nejvyssi id ve vlakne
sub search_supid {
   my ($pkg_p, $forum,$id)=@_;
   my $sth=$dbh->prepare(q{
      select
         Forum.supid
      from
         Komunikace.Forum
      where
         Forum.forum=? and
         Forum.id=?
   });
   $sth->execute($forum,$id);
   my $ref=$sth->fetchrow_hashref;
   $sth->finish;
   if(length($$ref{supid})) {
      $id=$pkg->search_supid($forum,$$ref{supid});
   }
   return $id;
}

#-- vraci vlastnika souboru
sub fileowner {
   my ($pkg_p,$file)=@_;
   my ($dev,$ino,$mode,$nlink,$uid)=stat($file);
   return getpwuid($uid);
}

#-- vraci datum posledni modifikace souboru
sub filemdate {
   my ($pkg_p,$file)=@_;
   my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime)=stat($file);
   return time2str("%Y-%m-%d", $mtime);
}

#--- Urceni barvy jakym bude tisten predmet (novy modre jinak cerne) ---
sub print_subject {
   my ($pkg_p, $tO,$message,$highlite)=@_;
   my ($y1,$m1,$d1)=split(/-/,substr($$message{datzapis},0,10));
   my $subject_color;
   if(Date::Calc::Delta_Days(($y1,$m1,$d1),Date::Calc::Today())<=7) {
      $subject_color="blue";
   }
   else {
      $subject_color="black";
   }
   my $subject_highlite;
   if(defined($$message{punc}) and $highlite) {
      $subject_highlite=$punc_highlight{$$message{punc}};
   }
   if(!defined($$message{punc})) {
      $tO=$tO->aT("strong");
   }
   $tO->aT(
      "a",
      {
         href => $pkg->forumpath . "show.phtml?forum=" . encode_uri($$message{forum}) . "&id=$$message{id}",
         style => "text-decoration: none; color: $subject_color; background-color: $subject_highlite"
      },
      $$message{predmet}
   );
}

#--- Tisteni puncu v prislusne barve ---
sub print_punc {
   my ($pkg_p, $tO,$punc,$puncuser)=@_;
   $punc="\x{a0}" if(!defined($punc)); # tvrda mezera
   $tO=$tO->aT("font", { color => $punc_color{$punc} } );
   if($punc eq $puncuser) {
      $tO=$tO->aT("strong");
   }
   my $style;
   if($punc eq $puncuser) {
      $style="font-size: 125%";
   }
   else {
      $style="font-size: 70%";
   }
   $tO->aT("span", { style => $style }, $punc);
}

#- fce vraci jmeno fora
sub forum_name {
   my ($pkg_p, $forum)=@_;
   my $forum_name;
   #- pokud existuje soubor jde o forum k dokumentu a zde zjistis jeho title
   if(-e($pkg->webpath . $forum)) {
      $forum_name=$pkg->htmltitle($pkg->webpath . $forum);
      if(!length($forum_name)) {
         $forum_name="beze jmna";
      }
   }
   # jinak nejde o forum k dokumentu a zjisti jeho pripadne pojmenovani
   else {
      $forum_name=My->okruh_nazev($forum);
   }
   return $forum_name;
}

#- fce vraci cely plaintext nazev fora
sub forum_name_title {
   my ($pkg_p,$forum,@id)=@_;
   my $title;
   my $xhtmlO=$pkg->forum_name_xhtml($forum);
   $title=$xhtmlO->findvalue(".");
   # konverze cestiny
   $title=Cs->utf2cs($title,"windows-1250"); # predpokladam, ze vnitrne pracuji s windows-1250
   # pokud je zobrazovan jen jeden prispevek - dej do title i jeho subject
   if(defined($id[0]) and $#id==0) {
      my $message=$pkg->read_message($forum,$id[0]);
      $title.=" - " . $$message{predmet};
   }
   return $title;
}

#- fce vraci cely nazev fora s pripadnym linkem na dokument (u fora k dokumentu)
sub forum_name_xhtml {
   my ($pkg_p, $forum)=@_;
   my $docxhtml=XML::LibXML::Document->new;
   my $xhtmlO=MyXML->new("xhtml");
   $docxhtml->setDocumentElement($xhtmlO);
   my $forum_name_extfce={
      "/Vyroba_priprava/Zakazky/Forum/" => {
         output => "Output->outputzak",
         info => "Info->zak"
      },
      "/Vyroba_priprava/Material/Forum/" => {
         output => "Output->outputmat",
         info => "Info->mat"
      },
      "/Vyroba_priprava/Naradi/Forum/" => {
         output => "Output->outputnar",
         info => "Info->nar"
      },
      "/Spolecnost/Str/Forum/" => {
         output => "Output->outputstr",
         info => "Info->str"
      },
      "/Spolecnost/Zamestnanci/Forum/" => {
         output => "Output->outputoc",
         info => "Info->oc"
      }
   };
   my ($forum_dir,$forum_val);
   if($forum=~/^(.*\/)([^\/]*)$/) {
      $forum_dir=$1;
      $forum_val=$2;
   }
   if(defined($$forum_name_extfce{$forum_dir})) {
      $xhtmlO->aT("br");
      my ($pkg_output,$output_fce)=split(/\-\>/,$$forum_name_extfce{$forum_dir}{output});
      my ($pkg_info,$info_fce)=split(/\-\>/,$$forum_name_extfce{$forum_dir}{info});
      $pkg_output->$output_fce($xhtmlO, $pkg_info->$info_fce($forum_val) );
   }
   else {
      my $forum_name=$pkg->forum_name($forum);
      if(-e($pkg->webpath . $forum)) {
         my $doc_link=$forum; $doc_link=~s/\.fhtml$//;
         $xhtmlO->aT(undef,{},"Forum k dokumentu ");
         $xhtmlO->aT("br");
         $xhtmlO->aT(
            "a",
            {
               href => $doc_link,
               style => "text-decoration: none;"
            },
            $forum_name
         );
      }
      else {
         $xhtmlO->aT(undef,{},$forum_name);
      }
   }
   return $xhtmlO;
}

#-- fce zjistujici title u html souboru
sub htmltitle {
   my ($pkg_p,$file)=@_;
   require HTTP::Headers;
   require HTML::HeadParser;
   open(FILE,$file);
   my $doc;
   while(my $radek=<FILE>) {
      $doc.=$radek;
   }
   close(FILE);
   my $h = HTTP::Headers->new;
   my $p = HTML::HeadParser->new($h);
   $p->parse($doc);
   undef $p;
   return $h->title;
}

sub set_limit {
   my ($pkg_p,$limit0,$limit1)=@_;
   $limit0*=1; $limit1*=1;
   if($limit0<1) {
      $limit0=1;
   }
   if($limit1<1) {
      $limit1=20;
   }
   return ($limit0,$limit1);
}


#=== Hlavni vystupni fce - drive byly soucasti izolovanych souboru

#-- zobrazeni fora
sub forum_show {
   my ($pkg_p, $tO,$query)=@_;
   #--- Definice vstupnich promennych
   my $forum=$query->param('forum');
   my @id=$query->param('id');
   my $limit0=$query->param('limit0');
   my $limit1=$query->param('limit1');
   my $mode=$query->param('mode');
   my $uziv=$query->param('uziv');
   my $puncmin=$query->param('puncmin');
   #--- Pripadne korekce zadanych limitu
   ($limit0,$limit1)=$pkg->set_limit($limit0,$limit1);
   #--- Pokud nebyl zadan mode vnutim time
   if(!defined($mode)) {
      $mode="time";
   }
   #- Pokud je zadano jedno id a mode je thread pokusis se vyhledat nejvyssi id ve vlakne
   if($#id==0 and $mode=~/thread/) {
      $id[0]=$pkg->search_supid($forum,$id[0]);
   }
   #-- pokud jde o globalnejsi vypis bez urceni konkretnich zprav, zobraz formular na moznosti filtrovani a meneni zobrazeni
   if(!defined(@id) or ($#id==0 and $mode=~/thread/)) {
      my %modetext=(
         "time" => "znn dle asu",
         "thread" => "znn dle nit",
         "indextime" => "strun dle asu",
         "indexthread" => "strun dle nit"
      );
      my $modemenu;
      if(defined(@id)) { # nejde o zobrazeni nite z jednoho prispevku
         $modemenu=[
            ["thread",$modetext{thread}],
            ["indexthread",$modetext{indexthread}]
         ];
      }
      else {
         $modemenu=[
            ["time",$modetext{time}],
            ["thread",$modetext{thread}],
            ["indextime",$modetext{indextime}],
            ["indexthread",$modetext{indexthread}]
         ];
      }
      my $formprvky;
      $$formprvky{forum}={ type => "hidden", value => $forum };
      $$formprvky{mode}={ print => "zobrazen", type => "select", selectmenu => $modemenu, value => $mode };
      delete($$formprvky{_layout});
      push(@{$$formprvky{_layout}[0][0]{names}}, ("forum","mode"));
      if(defined(@id)) { # jde o zobrazeni nite z jednoho prispevku
         $$formprvky{id}={ type => "hidden", value => $id[0] };
         push(@{$$formprvky{_layout}[0][0]{names}}, ("id"));
      }
      else {
         $$formprvky{limit0}={ print => "od pspvku", value => $limit0, size => 4 };
         $$formprvky{limit1}={ print => "poet pspvk", value => $limit1, size => 4 };
         $$formprvky{uziv}={ print => "jen pspvky uivatele", value => $uziv, size => 8 };
         my $puncmenu=[
            ["","libovolnm"],
            ["0",">=0"],
            ["1",">=1"],
            ["2",">=2"],
            ["3","=3"]
         ];
         $$formprvky{puncmin}={ print => "jen pspvky s puncem", type => "select", selectmenu => $puncmenu, value => $puncmin };
         push(@{$$formprvky{_layout}[0][0]{names}}, ("limit0","limit1"));
         push(@{$$formprvky{_layout}[0][1]{names}}, ("uziv","puncmin"));
      }
      Form->form_val_list(
         $tO,
         $formprvky,
         ["Provst zmnu zobrazen"]
      );
   }
   $tO->aT("hr");
   #-- nabydni link na moznost pridat dalsi prispevek
   my $pO=$tO->aT("p");
   $pO->aT(
      "a",
      { href => $pkg->forumpath . "add.phtml" . "?forum=" . encode_uri($forum) },
      "Vloit nov pspvek"
   );
   #-- prihlasit / odhlasit Forum k odberu
   my $odber=$pkg->odber;
   if(!defined($$odber{$forum})) {
      my $pO=$tO->aT("p");
      my $sth_odber=$dbh->prepare(q{
         select
            Forum_odber.uzivzapis
         from
            Komunikace.Forum_odber
         where
            Forum_odber.uzivzapis=? and
            Forum_odber.forum=?
      });
      $sth_odber->execute(My->remote_user,$forum);
      if($sth_odber->rows<1) {
         $pO->aT(
            "a",
            { href => $pkg->forumpath . "show.phtml?_idcs=" . encode_uri(get_env("idcs")) . "&setodber=" . encode_uri("1") },
            "Pihlsit forum k odbru"
         );
      }
      else {
         $pO->aT(
            "a",
            { href => $pkg->forumpath . "show.phtml?_idcs=" . encode_uri(get_env("idcs")) . "&setodber=" . encode_uri("0") },
            "Odhlsit forum z odbru"
         );
      }
      $sth_odber->finish;
   }
   #-- zobraz prispevky
   my $pO=$tO->aT("p");
   $pkg->show_message($pO,$forum,$mode,$limit0,$limit1,$uziv,$puncmin,@id);
}

#-- nastaveni puncu
sub set_punc {
   my ($pkg_p, $forum,$id,$punc)=@_;
   #--- opuncovani
   my $sth=$dbh->prepare(q{
      replace
         Komunikace.Forum_punc
      (
         Forum_punc.forum,
         Forum_punc.id,
         Forum_punc.punc,
         Forum_punc.datzapis,
         Forum_punc.uzivzapis
      )
      values (?,?,?,?,?)
   });
   $sth->execute($forum,$id,$punc,My->datetime_now,My->remote_user);
}

#-- nastaveni odberu
sub set_odber {
   my ($pkg_p, $forum,$odber)=@_;
   if($odber) {
      my $sth=$dbh->prepare(q{
         replace
            Komunikace.Forum_odber
         (
            Forum_odber.forum,
            Forum_odber.datzapis,
            Forum_odber.uzivzapis
         )
         values (?,?,?)
      });
      $sth->execute($forum,My->datetime_now,My->remote_user);
   }
   else {
      my $sth=$dbh->prepare(q{
         delete from
            Komunikace.Forum_odber
         where
            Forum_odber.forum=? and
            Forum_odber.uzivzapis=?
      });
      $sth->execute($forum,My->remote_user);
   }
}


#=== Pomocne vystupni fce

#-- fce vytvari html dokument urceny pro forum
sub creahtml {
   my ($pkg_p, $outputdev,$idcs,$query)=@_;
   set_env("title"," - ".$pkg->forum_name_title($query->param('forum'),$query->param('id')));
   my ($docO,$htmlO,$bodyO)=MyXML->creahtml($outputdev,$idcs);
   # zmena h1 nadpisu
   if(my ($h1O) = $bodyO->findnodes("h1")) {
      $bodyO->removeChild($h1O);
      my $h1O=$bodyO->aT("h1",{}, My->okruh_nazev . " - " );
      $h1O->aNchilds($pkg->forum_name_xhtml($query->param('forum')));
   }
   # zmena paticky - posledni link nect vzdy ukazzuje na zakladni zobrazeni fora a nese take nazev fora
   foreach my $aO ($bodyO->findnodes('//div/a')) {
      my $aOhref=$aO->findvalue('@href');
      if($aOhref=~/(show\.phtml|add\.phtml)/) {
         my $docxhtml=XML::LibXML::Document->new;
         my $anewO=MyXML->new("a");
         $docxhtml->setDocumentElement($anewO);
         $anewO->aT(undef,{},$pkg->forum_name_title($query->param('forum')));
         $anewO->sA("href", $pkg->forumpath . "show.phtml?forum=" . encode_uri($query->param('forum')));
         my $attrs=$aO->attributes;
         foreach my $attr (keys %{$$attrs{NodeMap}}) {
            if($attr ne "href") {
               my $value = $$attrs{NodeMap}{$attr}->value;
               $anewO->sA($attr, $value);
            }
         }
         $aO->replaceNode($anewO);
         last;
      }
   }
   return ($docO,$htmlO,$bodyO);
}

#-- fce zjistuje zda ma uzivatel na forum pravo - pokud ne, program je ukoncen
sub pravo {
   my ($pkg_p, $query)=@_;
   if(!defined($query->param('forum')) or !My->pravo(My->remote_user,$query->param('forum'))) {
      my $idcs=$query->param('_idcs');
      # nastav promenou obsahujici popis zarizeni na ktere bude probihat vystup
      my $outputdev=My->outputdev($query);
      $$outputdev{bezpre}=1;
      # vytvoreni html dokumentu
      my ($docO,$htmlO,$bodyO)=$pkg->creahtml($outputdev,$idcs,$query);
      $bodyO->aT("p",{},"Forum nemohu zobrazit z dvodu nedostatench prv.");
      Output->outputprint($docO,$outputdev);
      exit;
   }
}


return 1;

